# --------------------
# Imports
# --------------------
importFrom(survival, coxph)
importFrom(survival, survfit)
importFrom(survival, Surv)
importFrom("stats", "coef", "lm", "pnorm", "sd", "vcov", "runif", "var")

# --------------------
# Exports
# --------------------
export(wintime, markov, km, COMP)
